import {
  AudioPlayerStatus,
  createAudioPlayer,
  createAudioResource,
  VoiceConnectionStatus,
} from "@discordjs/voice";
import { Command } from ".";
import { existsSync } from "node:fs";

const cmdPlay: Command = {
  names: ["play", "soundtest"],
  desc: "test voice features",
  async checkPerm(msg) {
    return msg.author.id === "778441081883983893";
  },
  async handler(msg, { reply, say }) {
    const voiceState = msg.member?.voiceState;
    if (!voiceState?.channelID)
      return await reply("You are not in a voice channel");

    if (msg.guildID !== null && msg.client.getVoiceConnection(msg.guildID)) {
      return await reply("I am already connected");
    }

    const conn = voiceState.channel?.join({
      selfMute: false,
    });
    if (!conn) return await reply("Failed to join channel");

    conn.on(VoiceConnectionStatus.Disconnected, () => {
      console.log("Rejoining from disconnection!!");
      conn.rejoin();
    });

    const player = createAudioPlayer();
    conn.subscribe(player);

    conn.on("stateChange", (oldSt, newSt) => {
      console.log(`Moving from ${oldSt.status} to ${newSt.status}`);
    });
    conn.on("error", (err) => {
      console.error("Connection experienced error!", err);
    });

    player.once(AudioPlayerStatus.Playing, async () => {
      await say("Playing audio");
    });
    player.on(AudioPlayerStatus.Idle, async () => {
      await say("Audio finished playing");
    });

    const name = msg.content.match(/\s(.*)$/)?.[1];
    if (!name || !existsSync(name)) {
      return await reply("Audio file not found");
    }
    console.log(name);

    const audio = createAudioResource(name);
    player.play(audio);
  },
};

export default cmdPlay;
